package com.cesys.cebo.cebomsr;

import java.io.IOException;

/**
 * Offers functionality for counter periphery. Instances can be retrieved calling
 * getCounters() of the respective {@link Device} instance.
 *
 * @author Thomas Hoppe
 */
public final class Counter implements Input {
	/**
	 * The possible trigger configuration values.
	 */
	public enum CounterConfig {
		RisingEdge(SystemConnector.CeboMsrCounterConfigRisingEdge),
		FallingEdge(SystemConnector.CeboMsrCounterConfigFallingEdge),
		Alternating(SystemConnector.CeboMsrCounterConfigAlternating);

		private final int _counterConfigId;
		private CounterConfig(int counterConfigId) {
			_counterConfigId = counterConfigId;
		}

		static CounterConfig getById(int id) {
			switch (id) {
			case SystemConnector.CeboMsrCounterConfigRisingEdge: return RisingEdge;
			case SystemConnector.CeboMsrCounterConfigFallingEdge: return FallingEdge;
			case SystemConnector.CeboMsrCounterConfigAlternating: return Alternating;
			}
			throw new IllegalArgumentException();
		}
	}

	private final Device _device;
	private final int _interfaceId;
	private final int _index;
	private final String _name;

	int getInterfaceId() {
		return _interfaceId;
	}

	int getIndex() {
		return _index;
	}

	Counter(Device device, int interfaceId, int index) throws IOException {
		_device = device;
		_interfaceId = interfaceId;
		_index = index;
		byte[] tmp = new byte[SystemConnector.MaxIdentLength];
		SystemConnector.checkError(SystemConnector.CeboMsrGetPeripheralName(device.getHandle(), interfaceId, tmp));
		_name = Utils.convertByteArrayToString(tmp);
	}

	/**
	 * Reset counter to 0.
	 */
	public void reset() throws IOException {
		SystemConnector.checkError(SystemConnector.CeboMsrResetCounter(_device.getHandle(), getInterfaceId()));
	}

	/**
	 * Enable or disable counter.
	 * @param enable State to set.
	 */
	public void setEnabled(boolean enable) throws IOException {
		SystemConnector.checkError(SystemConnector.CeboMsrSetCounterEnable(_device.getHandle(), getInterfaceId(), enable ? 1 : 0));
	}

	/**
	 * Get current counter state.
	 */
	public boolean isEnabled() throws IOException {
		int[] r = new int[1];
		SystemConnector.checkError(SystemConnector.CeboMsrGetCounterEnable(_device.getHandle(), getInterfaceId(), r));
		return r[0] != 0;
	}

	/**
	 * Define behavior of the counter.
	 * @param counterConfig Use one of the values from Counter::CounterConfig.
	 */
	public void setConfig(CounterConfig counterConfig) throws IOException {
		SystemConnector.checkError(SystemConnector.CeboMsrSetCounterConfig(_device.getHandle(), getInterfaceId(), counterConfig._counterConfigId));
	}

	/**
	 * Request configuration of the counter.
	 * @return One of the values from Counter::CounterConfig.
	 */
	public CounterConfig getConfig() throws IOException {
		int[] r = new int[1];
		SystemConnector.checkError(SystemConnector.CeboMsrGetCounterConfig(_device.getHandle(), getInterfaceId(), r));
		return CounterConfig.getById(r[0]);
	}

	/**
	 * Read current counter value.
	 */
	public long read() throws IOException {
		long[] r = new long[1];
		SystemConnector.checkError(SystemConnector.CeboMsrReadCounter(_device.getHandle(), getInterfaceId(), r));
		return r[0];
	}

	/**
	 * Returns name of the component.
	 */
	public String getName() {
		return _name;
	}

	public String toString() {
		return "Counter" + _index;
	}
}
